@@ -25,6 +25,8 @@
 #include "lib/compress/compress.h"
 #include "lib/evloop/compat_libevent.h"
 #include "lib/crypt_ops/crypto_init.h"
+#include "lib/version/torversion.h"
+#include "app/main/subsysmgr.h"
 
 #include <stdio.h>
 #ifdef HAVE_FCNTL_H
@@ -230,12 +232,12 @@ void
 tinytest_prefork(void)
 {
   free_pregenerated_keys();
-  crypto_prefork();
+  subsystems_prefork();
 }
 void
 tinytest_postfork(void)
 {
-  crypto_postfork();
+  subsystems_postfork();
   init_pregenerated_keys();
 }
 
@@ -259,26 +261,15 @@ main(int c, const char **v)
   int loglevel = LOG_ERR;
   int accel_crypto = 0;
 
-  /* We must initialise logs before we call tor_assert() */
-  init_logging(1);
+  subsystems_init_upto(SUBSYS_LEVEL_LIBS);
 
-  update_approx_time(time(NULL));
   options = options_new();
-  tor_threads_init();
-  tor_compress_init();
-
-  network_init();
-
-  monotime_init();
 
   struct tor_libevent_cfg cfg;
   memset(&cfg, 0, sizeof(cfg));
   tor_libevent_initialize(&cfg);
 
   control_initialize_event_queue();
-  configure_backtrace_handler(get_version());
-
-  unsigned num=1, den=1;
 
   for (i_out = i = 1; i < c; ++i) {
     if (!strcmp(v[i], "--warn")) {
@@ -291,19 +282,6 @@ main(int c, const char **v)
       loglevel = LOG_DEBUG;
     } else if (!strcmp(v[i], "--accel")) {
       accel_crypto = 1;
-    } else if (!strcmp(v[i], "--fraction")) {
-      if (i+1 == c) {
-        printf("--fraction needs an argument.\n");
-        return 1;
-      }
-      const char *fracstr = v[++i];
-      char ch;
-      if (sscanf(fracstr, "%u/%u%c", &num, &den, &ch) != 2) {
-        printf("--fraction expects a fraction as an input.\n");
-      }
-      if (den == 0 || num == 0 || num > den) {
-        printf("--fraction expects a valid fraction as an input.\n");
-      }
     } else {
       v[i_out++] = v[i];
     }
@@ -327,6 +305,7 @@ main(int c, const char **v)
     s.masks[LOG_WARN-LOG_ERR] |= LD_BUG;
     add_callback_log(&s, log_callback_failure);
   }
+  flush_log_messages_from_startup();
   init_protocol_warning_severity_level();
 
   options->command = CMD_RUN_UNITTESTS;
@@ -363,54 +342,10 @@ main(int c, const char **v)
 
   atexit(remove_directory);
 
-  /* Look for TOR_SKIP_TESTCASES: a space-separated list of tests to skip. */
-  const char *skip_tests = getenv("TOR_SKIP_TESTCASES");
-  if (skip_tests) {
-    smartlist_t *skip = smartlist_new();
-    smartlist_split_string(skip, skip_tests, NULL,
-                           SPLIT_IGNORE_BLANK, -1);
-    int n = 0;
-    SMARTLIST_FOREACH_BEGIN(skip, char *, cp) {
-      n += tinytest_skip(testgroups, cp);
-      tor_free(cp);
-    } SMARTLIST_FOREACH_END(cp);
-    printf("Skipping %d testcases.\n", n);
-    smartlist_free(skip);
-  }
-
-  if (den != 1) {
-    // count the tests. Linear but fast.
-    unsigned n_tests = 0;
-    struct testgroup_t *tg;
-    struct testcase_t *tc;
-    for (tg = testgroups; tg->prefix != NULL; ++tg) {
-      for (tc = tg->cases; tc->name != NULL; ++tc) {
-        ++n_tests;
-      }
-    }
-    // Which tests should we run?  This can give iffy results if den is huge
-    // but it doesn't actually matter in practice.
-    unsigned tests_per_chunk = CEIL_DIV(n_tests, den);
-    unsigned start_at = (num-1) * tests_per_chunk;
-
-    // Skip the tests that are outside of the range.
-    unsigned idx = 0;
-    for (tg = testgroups; tg->prefix != NULL; ++tg) {
-      for (tc = tg->cases; tc->name != NULL; ++tc) {
-        if (idx < start_at || idx >= start_at + tests_per_chunk) {
-          tc->flags |= TT_SKIP;
-        }
-        ++idx;
-      }
-    }
-  }
-
   int have_failed = (tinytest_main(c, v, testgroups) != 0);
 
   free_pregenerated_keys();
 
-  crypto_global_cleanup();
-
   if (have_failed)
     return 1;
   else
